package io.skore.smooch_plugin.delegate

import io.flutter.plugin.common.MethodChannel
import io.smooch.core.*
import java.util.*

class ConversationDelegate(private val methodChannel: MethodChannel) : Conversation.Delegate {

    override fun onInitializationStatusChanged(initStatus: InitializationStatus?) {
        methodChannel.invokeMethod("initializationStatusChanged", initStatus?.name ?: "unknown")
    }

    override fun onSmoochHidden() {
        methodChannel.invokeMethod("smoochHidden", true)
    }

    override fun onPaymentProcessed(msgAction: MessageAction?, paymentStatus: PaymentStatus?) {
        methodChannel.invokeMethod("paymentProcessed", paymentStatus?.name ?: "unknown")
    }

    override fun onMessageSent(msg: Message?, msgUploadStatus: MessageUploadStatus?) {
        // Do nothing
    }

    override fun onLoginComplete(loginResult: LoginResult?) {
        methodChannel.invokeMethod("loginComplete", loginResult?.name ?: "unknown")
    }

    override fun onSmoochShown() {
        methodChannel.invokeMethod("smoochShown", true)
    }

    override fun onCardSummaryLoaded(summary: CardSummary?) {
        // Do nothing
    }

    override fun onSmoochConnectionStatusChanged(connectionStatus: SmoochConnectionStatus?) {
        methodChannel.invokeMethod("smoochConnectionStatusChanged", connectionStatus?.name ?: "unknown")
    }

    override fun onMessagesReceived(conversation: Conversation?, msgList: MutableList<Message>?) {
        // Do nothing
    }

    override fun onConversationEventReceived(conversationEvent: ConversationEvent?) {
        methodChannel.invokeMethod("conversationEventReceived", conversationEvent?.name ?: "unknown")
    }

    override fun onMessagesReset(conversation: Conversation?, msgList: MutableList<Message>?) {
        // Do nothing
    }

    override fun shouldTriggerAction(messageAction: MessageAction?): Boolean {
        messageAction?.apply {
            return when (type.toLowerCase(Locale.ENGLISH)) {
                "postback" -> true
                "link",
                "webview" -> {
                    methodChannel.invokeMethod("handleMessageAction", uri)
                    false
                }
                else -> {
                    uri.contains("viaapp_terms", true)
                }
            }
        }
        return true
    }

    override fun onLogoutComplete(logoutResult: LogoutResult?) {
        methodChannel.invokeMethod("logoutComplete", true)
    }

    override fun onUnreadCountChanged(conversation: Conversation?, unreadCount: Int) {
        methodChannel.invokeMethod("updateChatUnreadCount", unreadCount)
    }
}